import re
import subprocess
import os

class Domain:
    path_bind="/etc/bind/zones/db.icodevn.net"
    main_domain="icodevn"
    dict_record={}
    pattern = re.compile(r"^([@\w\-.]+)\s+IN\s+([A-Z]+)\s+([^\s;]+)")

    def __init__(self):
        if not os.path.exists(self.path_bind):
            open(self.path_bind, "a").close()
        with open(self.path_bind, "r") as file:
                print("File mode:", file.mode)
                for line in file:
                    line =line.strip()
                    if not line or line.startswith(";") or line.startswith("$"):
                        continue    
                    match  =self.pattern.match(line)
                    if match:
                        name,r_type,ip=match.groups()
                        if name =="@" and r_type =="NS":
                            continue
                        if name =="@":
                            self.dict_record[self.main_domain]=ip
                        self.dict_record[name]=ip
        print("Domain initialized")
    
    def add_record(self,subdomain, record_type,value):
        try:
            if subdomain in self.dict_record:
                raise Exception("subdomain already exists") 
            with open(self.path_bind, "a") as file:
                file.write(f"{subdomain} IN {record_type} {value}\n")
            self.dict_record[subdomain] = value
            subprocess.run(["systemctl","restart","bind9"],check=False)
        except Exception as e:
            raise Exception("handle file error")
           
           
    def delete_record(self,subdomain):
        try:
            if subdomain not in self.dict_record:
               raise Exception("subdomain not found")
            with open(self.path_bind,"r") as file:
                lines = file.readlines()
            with open(self.path_bind,"w") as file:
                for line in lines:
                    if subdomain not in line:
                        file.write(line)
            subprocess.run(["systemctl","restart","bind9"],check=False)
        except Exception as e:
            raise Exception("handle file error")
            
        


from setuptools import setup, find_packages

setup(
    name="levdomain",  
    version="1.0.6",
    author="somx",
    description="A simple FastAPI app",
    long_description_content_type="text/markdown",
    packages=["src"],  
    package_dir={"src": "src"},
    include_package_data=True,
    python_requires=">=3.7",
    install_requires=[
        "fastapi",
        "uvicorn",
        "pydantic",
        "pydantic-settings",
        "requests",
        "python-dotenv"
    ],
    entry_points={
        "console_scripts": [
            "levdomain=src.app:main"
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)


import jwt
import datetime
from fastapi import HTTPException


class ApiKey:
    def __init__(self,secret):
        self.key=secret
    def generateKey(self):
        payload = {
            "server":"dns"
        }
        token = jwt.encode(payload,self.key,algorithm="HS256")
        return token
    def verifyToken(self,token):
        try:
            decoded = jwt.decode(token, self.key, algorithms=["HS256"])
            return decoded  # Trả về payload nếu hợp lệ
        except jwt.ExpiredSignatureError:
            raise HTTPException(status_code=401, detail="Token expired")
        except jwt.InvalidTokenError:
            raise HTTPException(status_code=401, detail="Invalid token")


from fastapi import FastAPI, File, UploadFile,Request, HTTPException,Query
from fastapi.responses import JSONResponse, FileResponse
import uvicorn
from src import domain, apiKey
import argparse
import os
from dotenv import load_dotenv

load_dotenv()

app =FastAPI()
domain =domain.Domain()
key=apiKey.ApiKey(os.getenv("SECRET_KEY"))


@app.middleware("http")
async def auth_middleware(request: Request, call_next):
    # Chỉ bảo vệ 2 route
    if request.url.path in ["/add_record", "/delete_record"]:
        auth_header = request.headers.get("Authorization")
        
        if not auth_header or not auth_header.startswith("Bearer "):
            return JSONResponse(
                status_code=401,
                content={"status": "error", "message": "Missing or invalid Authorization header"}
            )

        token = auth_header.split(" ")[1]

        try:
            key.verifyToken(token)
        except HTTPException as e:
            return JSONResponse(status_code=e.status_code, content={"status": "error", "message": e.detail})

    response = await call_next(request)
    return response

@app.post("/add_record")
def add_record(
    subdomain: str = Query(..., min_length=1),
    record_type: str = Query(..., min_length=1),
    value: str = Query(..., min_length=1)
):
    try:
        domain.add_record(subdomain, record_type, value)
        return JSONResponse(
            content={
                "status": "success",
                "message": "Record added successfully",
                "subdomain": subdomain
            },
            status_code=200
        )
    except Exception as e:
        return JSONResponse(
            content={
                "status": "error",
                "message": f"Failed to add record: {str(e)}",
                "subdomain": subdomain
            },
            status_code=400
        )


@app.delete("/delete_record")
def delete_record(subdomain: str = Query(..., min_length=1)):
    try:
        domain.delete_record(subdomain)
        return JSONResponse(
            content={
                "status": "success",
                "message": "Record deleted successfully",
                "subdomain": subdomain
            },
            status_code=200
        )
    except Exception as e:
        return JSONResponse(
            content={
                "status": "error",
                "message": f"Failed to delete record: {str(e)}",
                "subdomain": subdomain
            },
            status_code=400
        )


def main():
    print("Starting server...")
    parser = argparse.ArgumentParser(description="Domain CLI Tool")
    parser.add_argument("command", nargs="?", help="Command to run: gen-key")
    args = parser.parse_args()

    if args.command == "gen-key":
        print("🔑 Generated key:")
        print(key.generateKey()) 
    else:
        print("🚀 Starting FastAPI server...")
        uvicorn.run("levdomain:app", host="localhost", port=7000, reload=False)

if __name__ == "__main__":
    main()